/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <gnome.h>
#include <pthread.h>
#include <errno.h>

//#include "../uulib/uudeview.h"
//#include "../uulib/fptools.h"

#include "pan.h"

extern int stop_all;

/***************************************************************
* UUdeview Lib Stuff 
***************************************************************/

void
UUD_message_callback (void *opaque, char *msg, int level)
{
	if ((level == UUMSG_WARNING) || (level == UUMSG_ERROR)) {
		pan_error_dialog(_("An error occured during decoding.\n%s"), msg);
	}
}

int
UUD_busy_callback (void *opaque, uuprogress * progress)
{
	gui_set_progress_activity(progress->percent);
	if (stop_all)
		return -1;
	else
		return 0;
}

char *
UUD_file_name_filter (void *opaque, char *fname)
{
	char *ptr;

	if (fname == NULL)
		return NULL;

	/*
	 * strip directory information
	 */

	if ((ptr = _FP_strrchr(fname, '/')) != NULL)
		return ptr + 1;
	else if ((ptr = _FP_strrchr(fname, '\\')) != NULL)
		return ptr + 1;

	return fname;
}

int decode(article_data * adata, GSList *list)
{
	char path[PATH_MAX];
	uulist *item;
	GSList *p;
	static int count = 0;
	int rc = 0;

	chdir(props_get_download_dir());
	UUInitialize();
	UUSetMsgCallback(adata, UUD_message_callback);
	UUSetBusyCallback(adata, UUD_busy_callback, 1);
	UUSetFNameFilter(adata, UUD_file_name_filter);
	for (p = list; p; p = p->next) {
		g_snprintf(path, PATH_MAX, "/%s/k%s.body", props_get_data_dir(), (char *)p->data);
		if ((UULoadFile(path, NULL, 0)) != UURET_OK) {
			pan_error_dialog(_("Decoded couldn't load temporary file:\n%s"), path);
			UUCleanUp();
			for (p = list; p; p = p->next) {
				acache_delete(p->data);
				g_free(p->data);
			}
			g_slist_free(list);
			return -1;
		}
		count++;
	}
	if ((item = UUGetFileListItem(0)) != NULL) {
		if ((item->state & UUFILE_OK) != 0) {
			if ((UUDecodeFile(item, NULL)) != UURET_OK) {
				//pan_error_dialog(_("Error decoding '%s'"), adata->subject);
				gui_set_info_va(_("Error decoding '%s'"), adata->subject);
				rc = -1;
			} else {
				gui_set_info_va(_("Successfully decoded '%s'"), UUFNameFilter(item->filename));
				log_add_va("Successfully decoded '%s'", UUFNameFilter(item->filename));
				rc = 0;
			}
		} else {
			//pan_error_dialog(_("Error decoding '%s'"), adata->subject);
			gui_set_info_va(_("Error decoding '%s'"), adata->subject);
			rc = -1;
		}
	}
	UUCleanUp();
	for (p = list; p; p = p->next) {
		acache_delete(p->data);
		g_free(p->data);
	}
	g_slist_free(list);
	return rc;
}
